        /* Copyright (c) 2007 by IBM Corporation */

dojo.provide("zero.form.widget.field.NumberFieldMixin");

dojo.require("zero.form.common.components.statics");
dojo.require("zero.form.widget.field.TextFieldValidationMixin");
dojo.require("zero.form.widget.field.properties.WidthPropertyMixin");
dojo.require("zero.form.widget.field.properties.RangePropertyHandlerMixin");

dojo.declare("zero.form.widget.field.NumberFieldMixin", 
    [zero.form.widget.field.TextFieldValidationMixin], 
	{
		widthProperty: "",
        maxValue: '',
        minValue: '',
		numberType: TYPE_PROPERTY_DECIMAL,

		getHandlers: function()
		{
			return [new zero.form.widget.field.properties.WidthPropertyMixin(),
					new zero.form.widget.field.properties.RangePropertyHandlerMixin()];
		},
		
		startup: function()
	   	{
            zero.form.widget.field.NumberFieldMixin.superclass.startup.apply(this, arguments);
			this.setNumberType(this.numberType);
	   	},
		
		setNumberType: function(type)
		{
			this.numberType = type;
			this.dijitPlaces = (type == TYPE_PROPERTY_INTEGER)?0:undefined;
		},
		
		getNumberType: function()
		{
			return this.numberType;
		},

		toNumber: function(value)
		{
			if(value === "")
				return "";
			value = (value)?value:"0";
			var retVal = (this.digitPlaces == 0)? parseInt(value) : parseFloat(value);
			if(isNaN(retVal))
				retVal = value;
			return retVal;
		},
			   
	    setValue: function(val)
	    {
            if(val === undefined || val === null)
				val = 0;
			if(isNaN(val))
				val = "";
			this._settingValue = true;
	   		arguments[0] = ""+this.format(val,this.constraints);
			
			zero.form.widget.field.NumberFieldMixin.superclass.setValue.apply(this, arguments);
	    },
        
		setAdditionalProperties: function(pJson)
		{
			this.widthProperty = (pJson.width === undefined) ? "" : pJson.width;
			// empty string is the default min/max value - means unbounded			
			this.maxValue = (pJson.max === undefined) ? "" : pJson.max;
        	this.minValue = (pJson.min === undefined) ? "" : pJson.min;
			this.digitPlaces = pJson.digitPlaces;
        	this.numberType = (pJson.numberType === undefined) ? TYPE_PROPERTY_DECIMAL : pJson.numberType;
		},
		
		getAdditionalProperties: function(pJson)
		{
			if (this.widthProperty !== undefined && this.widthProperty !== "") 
			{
				pJson.width = this.widthProperty;
			}
			
			// don't set min/max if it is not defined
			if (this.maxValue !== undefined && this.maxValue !== "") 
			{
				pJson.max = this.maxValue;
			}
			if (this.minValue !== undefined && this.minValue !== "") 
			{
				pJson.min = this.minValue;
			}
			if (this.numberType !== undefined && this.numberType !== "") 
			{
				pJson.numberType = this.numberType;
			}
		},

        numericGetValue: function()
	    {
			//var val = zero.form.widget.field.NumberFieldMixin.superclass.getValue.apply(this, arguments);
	   	 	return this.format(this.toNumber(this.textbox.value),this.constraints);
	    },

		isContentValid: function()
		{
			var numOfDecPlaces = this.getNumOfDecimalDigits();
			if(this.digitPlaces && this.digitPlaces <numOfDecPlaces && isFinite(numOfDecPlaces))
			{
				return false;
			}
			return true;
		},

		getNumOfDecimalDigits: function()
		{
			var val = this.toNumber(this.textbox.value);
			if((typeof val === 'number' && !(isNaN(val))) || val === "")
			{
				var splittedNum = val.toString().split(".");
				if(splittedNum.length == 2)
				{
					return splittedNum[1].length;
				}
				return 0;
			}
			return Infinity;
		},
		
		validate: function()
		{
			if(this.digitPlaces)// >= this.getNumOfDecimalDigits())
			{
				var oldGetValue = this.getValue;
				this.getValue = this.numericGetValue;
				this.inherited("validate",arguments);
				this.getValue = oldGetValue;
			}
			else
				this.inherited("validate",arguments);
		},
		
		getWidth: function()
		{
			return this.widthProperty;
		},
		
		setWidth: function(pWidth)
		{
			this.widthProperty = pWidth;
		},
		
		getMax: function()
		{
			return this.maxValue;
		},
		
		setMax: function(pMax)
		{
			this.maxValue = pMax;
		},

		getMin: function()
		{
			return this.minValue;
		},
		
		setMin: function(pMin)
		{
			this.minValue = pMin;
		},

        // isValid added for bug 9585 isInRange check is needed
        isValid: function(/*Boolean*/ isFocused){
          return this.inherited(arguments) &&
          ((this._isEmpty(this.getValue()) && !this.required) || this.isInRange(isFocused)); // Boolean
        }
});
